<div class="entry">
  <h1><a href="/entry/{{ entry.slug }}">{{ escape(entry.title) }}</a></h1>
  <div class="date">{{ locale.format_date(entry.published, full_format=True, shorter=True) }}</div>
  <div class="body">{{ entry.html }}</div>
  {% if current_user and current_user.administrator %}
    <div class="admin"><a href="/compose?key={{ str(entry.key()) }}">{{ _("Edit this post") }}</a></div>
  {% end %}
</div>
